using System.Net;
using System.Net.Sockets;
using System.Text;

public class UDPServer
{
    private int listenPort;
    private UdpClient listener;
    private IPEndPoint groupEP;

    public UDPServer(int port)
    {
        // Construction
        listenPort = port;
        try
        {
            listener = new UdpClient(listenPort);
            groupEP = new IPEndPoint(IPAddress.Any, listenPort);
        }
        catch (Exception e)
        {
            Console.WriteLine(e.ToString());
            throw new ArgumentException(
                $"Not currect port num: {port}",
                nameof(port)
                );
        }
    }

    private void ReceiveHandler(IAsyncResult res)
    {
        var remoteEP = new IPEndPoint(IPAddress.Any, listenPort);
        byte[] bytes = listener.EndReceive(res, ref remoteEP);

        var sign = bytes[0];

        string recvData;
        if (sign == (byte)WebProtocol.PackSign.SendAddr)
        {
            var resData = WebProtocol.SendAddrPackage.FromBytes(bytes);
            if (resData.IsSuccessful)
                recvData = resData.Value.ToString();
            else
                recvData = resData.Error.ToString();
        }
        else if (sign == (byte)WebProtocol.PackSign.SendData)
        {
            recvData = "";
        }
        else if (sign == (byte)WebProtocol.PackSign.RecvData)
        {
            recvData = "";
        }
        else if (sign == (byte)WebProtocol.PackSign.RecvResp)
        {
            recvData = "";
        }
        else
        {
            recvData = Encoding.ASCII.GetString(bytes, 0, bytes.Length);
        }

        string remoteStr = (remoteEP is null) ? "Unknown" : $"{remoteEP.Address.ToString()}:{remoteEP.Port.ToString()}";
        Console.WriteLine($"Receive Data from {remoteStr} at {DateTime.Now.ToString()}:");
        Console.WriteLine($"Original Data: {BitConverter.ToString(bytes).Replace("-", " ")}");
        if (recvData.Length != 0) Console.WriteLine(recvData);
        Console.WriteLine();

        listener.BeginReceive(new AsyncCallback(ReceiveHandler), null);
    }

    public void Start()
    {
        try
        {
            listener.BeginReceive(new AsyncCallback(ReceiveHandler), null);
        }
        catch (Exception e)
        {
            Console.WriteLine(e.ToString());
        }
        finally
        {

        }
    }

    public void Stop()
    {
        listener.Close();
    }
}

